#!/usr/bin/env ruby

require 'bundler/cli'
require 'yaml'
require 'erb'
require 'thor'

require 'ezdrb'

module Ezdrb

  module Helpers

    def self.validate(var, error_msg = '')
      var = Thor::Shell::Basic.new.ask error_msg while var.empty? || var.nil?
      var
    end

    def self.load_template(template)
      ERB.new(File.read("#{Bundler.rubygems.find_name('ezdrb').first.full_gem_path}/lib/ezdrb/templates/#{template}.erb"), 0, '%<>')
    end

    class YamlHandler

      attr_reader :yamlfile, :path

      def initialize(file)
        @path = "#{file}.yml"
        @yamlfile = YAML.load_file(path)
      end

      def append_to_list(list, value)
        if !@yamlfile[list].include?(value) then
          @yamlfile[list].push(value)
        end
      end

      def write
        File.write(@path, @yamlfile.to_yaml)
      end

    end

  end
end

module Ezdrb

  class CLI < Thor

    desc 'init', 'Initializes a Discord bot template.'
    def init
      if File.file?('config/bot.yml') then
        say('You already have a Discord bot in here.')
      else
        say("Creating new EZDRB project...\n\n")

        prefix = ask('Set prefix:')
        prefix = Helpers::validate(prefix, 'Error: you must set a prefix:')

        token = ask('Set token:')
        token = Helpers::validate(token, 'Error: you must set a token:')

        begin
          Dir.mkdir('config')
          Dir.mkdir('commands')
          Dir.mkdir('events')

          File.open('config/bot.yml', 'w') do |file|
            template = Helpers::load_template('BotYml').result(binding)
            file.write(template)
          end

          File.open('config/commands.yml', 'w') do |file|
            file.write('commands: []')
          end

          File.open('config/events.yml', 'w') do |file|
            file.write('events: []')
          end

          File.open('Attributes.rb', 'w') do |file|
            template = Helpers::load_template('Attributes').result(binding)
            file.write(template)
          end

          File.open('Commands.rb', 'w') do |file|
            template = Helpers::load_template('Commands').result(binding)
            file.write(template)
          end

          File.open('Events.rb', 'w') do |file|
            template = Helpers::load_template('Events').result(binding)
            file.write(template)
          end

          File.open('bot.rb', 'w') do |file|
            template = Helpers::load_template('BotRun').result(binding)
            file.write(template)
          end
        rescue => e
          say('Something went wrong while creating the project.')
          say(" => #{e.message}")
        end
      end
    end

    desc 'command <command_name>', 'Creates a new bot command'
    def command(command_name)
      command_name = command_name.strip.downcase

      begin
        commands = YAML.load_file('config/commands.yml')
        commands = commands.values.flatten
      rescue => e
        say("ERROR: Couldn't read commands.yml")
        exit!
      end

      if commands.include?(command_name) then
        say("ERROR: #{command_name} already exists")
        exit!
      end

      begin
        commands_yaml = Helpers::YamlHandler.new('config/commands')
        commands_yaml.append_to_list('commands', command_name)
        commands_yaml.write

        File.open("commands/#{command_name.capitalize}.rb", 'w') do |file|
          template = Helpers::load_template('SingleCommand').result(binding)
          file.write(template)
        end
      rescue => e
        say('Something went wrong while creating the command.')
        say(" => #{e.message}")
      end
    end

    desc 'event <event>', 'Creates a new event handler'
    def event(event)
      event = event.strip.downcase
      official_events = [:ready, :disconnected, :heartbeat, :typing, :message_edit, :message_delete, :reaction_add, :reaction_remove, :reaction_remove_all, :presence, :playing, :channel_create, :channel_update, :channel_delete, :channel_recipient_add, :channel_recipient_remove, :voice_state_update, :member_join, :member_update, :member_leave, :user_ban, :user_unban, :server_create, :mention, :server_update, :server_delete, :server_emoji, :server_emoji_create, :server_emoji_delete, :private_message, :direct_message, :server_emoji_update, :raw, :unknown, :pm, :dm, :message]
      official_events.freeze

      if official_events.include?(event.to_sym) then
        begin
          events = YAML.load_file('config/events.yml')
          events = events.values.flatten
        rescue => e
          say("ERROR: Couldn't read events.yml")
          exit!
        end

        if events.include?(event) then
          say("ERROR: #{event} already exists")
          exit!
        end

        begin
          events_yaml = Helpers::YamlHandler.new('config/events')
          events_yaml.append_to_list('events', event)
          events_yaml.write

          File.open("events/#{event.capitalize}.rb", 'w') do |file|
            template = Helpers::load_template('SingleEvent').result(binding)
            file.write(template)
          end
        rescue => e
          say('Something went wrong while creating the command.')
          say(" => #{e.message}")
        end

      else
        say('This event does not exist.')
      end
    end

  end
end